#!/usr/bin/env bash
#!/bin/env bash

## Name: easy_pipeline.sh
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Ready-to-use pipeline, with no given GFF by default. Variables should be modified properly.

errorhandling() {
	echo -ne "\n"
	local parent_lineno="$1"
	local message="$2"
	local code="${3:-1}"
	if [[ -n "$message" ]]
	then
	    echo "Error on or near line ${parent_lineno}: ${message}; exiting with status ${code}"
	else
	    echo "Error on or near line ${parent_lineno}; exiting with status ${code}"
	fi
	exit "${code}"
}

echo -ne "Time - start:\t\t"
date

########################################
## VARIABLES
########################################
## main working directory 
PREFIX=~/toto
#PREFIX=/home/user/toto

## RNAprof source folder
SRC=~/RNAprof/src

## mapping description
DESC=desc.cfg

## annotation
GFF=
#GFF=TAIR10.gff

## using gffprof
#GFFPROF=
GFFPROF="--gffprof"

## library-type
#LIBTYPE=fr-unstranded
LIBTYPE=fr-firststrand
#LIBTYPE=fr-secondstrand

## sequencing type
SEQTYPE=
#SEQTYPE="--Pair"

## maximum insert size
MIS=
#MIS=1000

## maximum number of processes
MAXPROC=24

## maximum memory per process
MAXMEM=2G

########################################
## PIPELINE
########################################
if [ ! -d $SRC ]
then
    errorhandling $LINENO "Error: $SRC not found" 2
fi
if [ ! $GFF ]
then
    GFFPROF="--gffprof"
fi

PYTHON=python

$SHELL $SRC/configure.sh --prefix=$PREFIX --description=$DESC --library-type=$LIBTYPE --gff=$GFF --MaxInsertSize=$MIS --CPU=$MAXPROC --memory=$MAXMEM $SEQTYPE > 1configure.out 2> 1configure.err
err=$?
if [ $err -ne 0 ]
then
    errorhandling $LINENO "configure.sh error" $err
fi
echo -ne "Time - configure:\t"
date

$SHELL $SRC/preprof.sh --prefix=$PREFIX --CPU=$MAXPROC $SEQTYPE > 2preprof.out 2> 2preprof.err
err=$?
if [ $err -ne 0 ]
then
    errorhandling $LINENO "preprof.sh error" $err
fi
echo -ne "Time - preprof:\t\t"
date

if [ "$GFFPROF" ]
then
    $SHELL $SRC/GFFprof.sh --prefix=$PREFIX --CPU=$MAXPROC --gff=$GFF > 3GFFprof.out 2> 3GFFprof.err
    err=$?
    if [ $err -ne 0 ]
    then
	errorhandling $LINENO "GFFprof.sh error" $err
    fi
    echo -ne "Time - GFFprof:\t\t"
    date
fi

$SHELL $SRC/coverage.sh --prefix=$PREFIX --CPU=$MAXPROC --memory=$MAXMEM $GFFPROF > 4coverage.out 2> 4coverage.err
err=$?
if [ $err -ne 0 ]
then
    errorhandling $LINENO "coverage.sh error" $err
fi
echo -ne "Time - coverage:\t"
date

$SHELL $SRC/RNAprof.sh --prefix=$PREFIX --CPU=$MAXPROC > 5RNAprof.out 2> 5RNAprof.err
err=$?
if [ $err -ne 0 ]
then
    errorhandling $LINENO "RNAprof.sh error" $err
fi
echo -ne "Time - RNAprof:\t\t"
date

i=0
for res in $PREFIX/result/*
do
    $PYTHON $SRC/python/report.py $res 
    let i++
    cp -R $SRC/js $res
    cp -R $SRC/css $res
done
echo -ne "Time - report:\t\t"
date
