#!/usr/bin/env perl
#!/bin/env perl

## Name: makeGFFrefine.pl
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret
## Description: Create scripts for refining GFF and separating strands

use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case);

my $help;
my $cluster;
my $path=".";
my $src;
my $gff;
my $feature;
my $memory="2G";
my $output="output.sh";
my $genome;
my $cpu=1;

my $instance;
my $gff_filename;
my $line;
my $perlsrc;
my @args;
my @chromosomes;

usage() if (($#ARGV<0) or !(GetOptions (
				'help|?' => \$help,
				'cluster' => \$cluster,
				'path=s' => \$path,
				'src=s' => \$src,
				'feature=s' => \$feature,
				'GenomeSize=s' => \$genome,
				'gff=s' => \$gff,
				'memory=s' => \$memory,
				'CPU=i' => \$cpu,
				'output=s' => \$output
			    ))
	    or defined($help));

$perlsrc = $src."/perl";

open(OUT, ">$output");

if (! -s $gff) {
    print OUT "echo \"Done: No given annotation\"\n";
    close(OUT);
    exit 0;
}
$gff_filename = &get_gff_name($gff);

if ($cluster) {
    print OUT "#\$ -S /bin/bash\n";
    print OUT "#\$ -N runGFFrefine\n";
    print OUT "#\$ -V\n";
    print OUT "#\$ -cwd\n";
    print OUT "#\$ -pe parallel ".$cpu."\n";
    print OUT "#\$ -l h_vmem=".$memory."\n";
    print OUT "\n";
}
else {
    print OUT "#!/usr/bin/env bash\n";
    print OUT "#!/bin/env bash\n";
    print OUT "\n";
}

print OUT "##variables\n";
print OUT "SRC=$perlsrc\n";
print OUT "DIR=$path\n";
print OUT "GFF=$gff_filename\n";
print OUT "FEATURE=$feature\n\n";

print OUT "##change working directory to rnaseq\n";
print OUT "cd \$DIR/rnaseq\n\n";

print OUT "##correct GFF format \n";
print OUT "cat $gff | awk \'{if ((\$3 ==\"gene\") || (\$3==\"transcript\") || (\$3==\"exon\")) print \$0}\' | \$SRC/patchisoform.pl -i - -o $gff".".patch\n\n";

print OUT "##separate GIVEN annotated gff into NEG and POS, sort and fusion the overlapped genes (fast)\n";
print OUT "echo \"\$SRC/gettsc.pl -i $gff".".patch -f \$FEATURE --strand - -o \$DIR/genome/NEG_\$GFF\" > \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"\$SRC/getisoforms.pl -i \$DIR/genome/NEG_\$GFF -o \$DIR/genome/NEG_iso_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"\$SRC/separatetscexon.pl -i \$DIR/genome/NEG_\$GFF -t \$DIR/genome/NEG_rawtsc_\$GFF --exon NEG_exons.list\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"rm -f NEG_exons.list\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"\$SRC/separatetscexon.pl -i \$DIR/genome/NEG_iso_\$GFF -t \$DIR/genome/NEG_tsc_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"sort -k1,1 -k4,4n -k5,5n \$DIR/genome/NEG_tsc_\$GFF \> \$DIR/genome/NEG_tsc_sort_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"\$SRC/fusionstrandtsc.pl -i \$DIR/genome/NEG_tsc_sort_\$GFF --prefix UNS --form --note -o \$DIR/genome/NEG_tsc_fusion_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"\$SRC/jointscexon.pl -i \$DIR/genome/NEG_tsc_fusion_\$GFF -o \$DIR/genome/NEG_fusion_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"rm \$DIR/genome/NEG_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"rm \$DIR/genome/NEG_tsc_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n";
print OUT "echo \"rm \$DIR/genome/NEG_tsc_sort_\$GFF\" >> \$DIR/script/gffrefine_neg.sh\n\n";

print OUT "echo \"\$SRC/gettsc.pl -i $gff".".patch -f \$FEATURE --strand + -o \$DIR/genome/POS_\$GFF\" > \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"\$SRC/getisoforms.pl -i \$DIR/genome/POS_\$GFF -o \$DIR/genome/POS_iso_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"\$SRC/separatetscexon.pl -i \$DIR/genome/POS_\$GFF -t \$DIR/genome/POS_rawtsc_\$GFF --exon POS_exons.list\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"rm -f POS_exons.list\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"\$SRC/separatetscexon.pl -i \$DIR/genome/POS_iso_\$GFF -t \$DIR/genome/POS_tsc_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"sort -k1,1 -k4,4n -k5,5n \$DIR/genome/POS_tsc_\$GFF \> \$DIR/genome/POS_tsc_sort_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"\$SRC/fusionstrandtsc.pl -i \$DIR/genome/POS_tsc_sort_\$GFF --prefix UNS --form --note -o \$DIR/genome/POS_tsc_fusion_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"\$SRC/jointscexon.pl -i \$DIR/genome/POS_tsc_fusion_\$GFF -o \$DIR/genome/POS_fusion_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"#rm \$DIR/genome/POS_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"#rm \$DIR/genome/POS_tsc_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n";
print OUT "echo \"#rm \$DIR/genome/POS_tsc_sort_\$GFF\" >> \$DIR/script/gffrefine_pos.sh\n\n";

print OUT "ls \$DIR/script/gffrefine_\*.sh | xargs -i --max-procs=$cpu bash -c \"bash {}\"\n";
print OUT "rm \$DIR/script/gffrefine_\*.sh\n";
print OUT "echo \"Done: separate GIVEN annotated gff into NEG and POS, sort and fusion the overlapped genes\"\n\n";

print OUT "##join all transcripts from given annotation (fast)\n";
print OUT "cat \$DIR/genome/NEG_tsc_fusion_\$GFF \$DIR/genome/POS_tsc_fusion_\$GFF | sort -k1,1 -k4,4n -k5,5n > \$DIR/genome/annotation_raw.gff\n";
print OUT "echo \"Done: join all transcripts from given annotation\"\n\n";

print OUT "##recover exon features (fast)\n";
print OUT "\$SRC/jointscexon.pl -i \$DIR/genome/annotation_raw.gff -o \$DIR/genome/annotation.gff\n";
print OUT "echo \"Done: recover exon features\"\n\n";

open(CHR, "$genome") or die("Genome size file not found\n");
@chromosomes = ();
while ($line = <CHR>) {
    chomp($line);
    @args = split("\t", $line);
    push(@chromosomes, $args[0]);
}
close(CHR);

print OUT "##separate transcripts and exons for each chromosome (fast)\n";
for (my $i=0; $i<=$#chromosomes; $i++) {
    print OUT "echo \"awk -v arg=$chromosomes[$i] \'\\\$1==arg\' \$DIR/genome/annotation_raw.gff \> gff/input_annotation_raw_$chromosomes[$i].gff\" > \$DIR/script/gffrefine_$chromosomes[$i].sh\n";
    print OUT "echo \"\$SRC/jointscexon.pl -i gff/input_annotation_raw_$chromosomes[$i].gff -e exon/input_exon_$chromosomes[$i].tsv -t gff/input_annotation_noexon_$chromosomes[$i].gff\" >> \$DIR/script/gffrefine_$chromosomes[$i].sh\n";
    print OUT "echo \"awk -v arg=- \'\\\$7==arg\' gff/input_annotation_noexon_$chromosomes[$i].gff \> gff/input_NEG_noexon_$chromosomes[$i].gff\" >> \$DIR/script/gffrefine_$chromosomes[$i].sh\n";
    print OUT "echo \"awk -v arg=+ \'\\\$7==arg\' gff/input_annotation_noexon_$chromosomes[$i].gff \> gff/input_POS_noexon_$chromosomes[$i].gff\" >> \$DIR/script/gffrefine_$chromosomes[$i].sh\n";
    print OUT "echo \"awk -v arg=. \'\\\$7==arg\' gff/input_annotation_noexon_$chromosomes[$i].gff \> gff/input_UNS_noexon_$chromosomes[$i].gff\" >> \$DIR/script/gffrefine_$chromosomes[$i].sh\n";
}
print OUT "ls \$DIR/script/gffrefine_\*.sh | xargs -i --max-procs=$cpu bash -c \"bash {}\"\n";
print OUT "rm \$DIR/script/gffrefine_\*.sh\n";
print OUT "rm output.gff\n";
print OUT "echo \"Done: separate transcripts and exons for each chromosome\"\n\n";

close(OUT);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: see configure.sh \n";
    exit 1;
}

## get gff filename
sub get_gff_name {
    my $gff = $_[0];
    my $lastslash = rindex($gff, "/");
    
    $gff = substr($gff, $lastslash+1);
    
    return $gff;
}
