#!/usr/bin/env perl
#!/bin/env perl

## Name: runtsccompare.pl
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# find non-coding transcript and their relative positions for all chromosomes
# --input, input description file
## eg.
## ref_NEG_tsc.gff
## ref_POS_tsc.gff
## new_NEG_tsc.gff
## new_POS_tsc.gff
## new_UNS_tsc.gff
# --genome, chromosome sizes in the genome
## eg.
## chr1 1000000
## chr2 1400000
# --src, source directory

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input = "tscfile.list";
my $genome = "species.genome";
my $src = ".";

my $line;
my $i;
my $perlsrc;
my @chromosomes;
my @files;
my @args;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "genome=s" => \$genome,
			 "src=s" => \$src)
	    or defined $help);

$perlsrc = $src."/perl";

if (-e "ncNEG.gff") {
    system("rm ncNEG.gff");
}
if (-e "ncPOS.gff") {
    system("rm ncPOS.gff");
}
if (-e "ncUNS.gff") {
    system("rm ncUNS.gff");
}
if (-e "asNEG.gff") {
    system("rm asNEG.gff");
}
if (-e "asPOS.gff") {
    system("rm asPOS.gff");
}
if (-e "undef.gff") {
    system("rm undef.gff");
}
if (-e "NEG_rnaseq.gff") {
    system("rm NEG_rnaseq.gff");
}
if (-e "POS_rnaseq.gff") {
    system("rm POS_rnaseq.gff");
}
if (-e "ncNEG_included.gff") {
    system("rm ncNEG_included.gff");
}
if (-e "ncPOS_included.gff") {
    system("rm ncPOS_included.gff");
}
if (-e "ncUNS_included.gff") {
    system("rm ncUNS_included.gff");
}
if (-e "ncNEG_overlapped.gff") {
    system("rm ncNEG_overlapped.gff");
}
if (-e "ncPOS_overlapped.gff") {
    system("rm ncPOS_overlapped.gff");
}
if (-e "ncUNS_overlapped.gff") {
    system("rm ncUNS_overlapped.gff");
}
if (-e "ncNEG_intergenic.gff") {
    system("rm ncNEG_intergenic.gff");
}
if (-e "ncPOS_intergenic.gff") {
    system("rm ncPOS_intergenic.gff");
}
if (-e "ncUNS_intergenic.gff") {
    system("rm ncUNS_intergenic.gff");
}

#<take chromosome list
open(CHR, "$genome") or die("Chromosome sizes not found\n");
@chromosomes = ();
while ($line = <CHR>) {
    chomp($line);
    @args = split(" ", $line);
    push(@chromosomes, $args[0]);
}
close(CHR);
#>

#<take filename list from tscfile.list 
open(TSC, "$input") or die("$input not found\n");
@files = ();
while ($line = <TSC>) {
    chomp($line);
    push(@files, $line);
}
close(TSC);

#>

for ($i=0; $i<=$#chromosomes; $i++) {
    #< separate chromosomes
    system("awk -v arg=$chromosomes[$i] \'\$1==arg\' $files[0] > NEG_tsc_$chromosomes[$i].gff");
    system("awk -v arg=$chromosomes[$i] \'\$1==arg\' $files[1] > POS_tsc_$chromosomes[$i].gff");
    system("awk -v arg=$chromosomes[$i] \'\$1==arg\' $files[2] > NEG_new_$chromosomes[$i].gff");
    system("awk -v arg=$chromosomes[$i] \'\$1==arg\' $files[3] > POS_new_$chromosomes[$i].gff");
    system("awk -v arg=$chromosomes[$i] \'\$1==arg\' $files[4] > UNS_new_$chromosomes[$i].gff");
    #>
    
    #< merge discovered NEG and annotated NEG
    system("$perlsrc/tsccomparechr.pl -i NEG_new_$chromosomes[$i].gff -r NEG_tsc_$chromosomes[$i].gff -o rnaseq_NEG_$chromosomes[$i].gff");
    #>
    
    #< merge discovered POS and annotated POS
    system("$perlsrc/tsccomparechr.pl -i POS_new_$chromosomes[$i].gff -r POS_tsc_$chromosomes[$i].gff -o rnaseq_POS_$chromosomes[$i].gff");
    #>
    
    #< extend NEG and POS with UNS
    system("$perlsrc/unscomparechr.pl -i UNS_new_$chromosomes[$i].gff -r rnaseq_NEG_$chromosomes[$i].gff -o rnaseq_UNS_NEG_$chromosomes[$i].gff");
    system("awk -v arg=. \'\$7==arg\' rnaseq_UNS_NEG_$chromosomes[$i].gff > rnaseq_UNS_$chromosomes[$i].gff");
    system("$perlsrc/unscomparechr.pl -i rnaseq_UNS_$chromosomes[$i].gff -r rnaseq_POS_$chromosomes[$i].gff -o rnaseq_UNS_POS_$chromosomes[$i].gff");
    system("awk -v arg=. \'\$7==arg\' rnaseq_UNS_POS_$chromosomes[$i].gff > rnaseq_UNS_$chromosomes[$i].gff");
    system("$perlsrc/tsccomparechr.pl --unstranded --extend 200 -i rnaseq_UNS_$chromosomes[$i].gff -r rnaseq_NEG_$chromosomes[$i].gff --nc nc_UNS_NEG_$chromosomes[$i].gff --antisense asUNSNEG_$chromosomes[$i].gff -o rnaseq_NEG_ext_$chromosomes[$i].gff");
    system("$perlsrc/tsccomparechr.pl --unstranded --extend 200 -i rnaseq_UNS_$chromosomes[$i].gff -r rnaseq_POS_$chromosomes[$i].gff --nc nc_UNS_POS_$chromosomes[$i].gff --antisense asUNSPOS_$chromosomes[$i].gff -o rnaseq_POS_ext_$chromosomes[$i].gff");
    #>
    
    #< find nc NEG, POS and UNS
    system("awk -v arg=- \'\$7==arg\' rnaseq_NEG_ext_$chromosomes[$i].gff > rnaseqNEG_$chromosomes[$i].gff");
    system("cat rnaseqNEG_$chromosomes[$i].gff >> NEG_rnaseq.gff");
    system("awk -v arg=+ \'\$7==arg\' rnaseq_POS_ext_$chromosomes[$i].gff > rnaseqPOS_$chromosomes[$i].gff");
    system("cat rnaseqPOS_$chromosomes[$i].gff >> POS_rnaseq.gff");
    
    system("$perlsrc/tsccomparechr.pl -i rnaseqNEG_$chromosomes[$i].gff -r NEG_tsc_$chromosomes[$i].gff --nc ncNEG_$chromosomes[$i].gff");
    system("cat ncNEG_$chromosomes[$i].gff >> ncNEG.gff");
    system("$perlsrc/tsccomparechr.pl -i rnaseqPOS_$chromosomes[$i].gff -r POS_tsc_$chromosomes[$i].gff --nc ncPOS_$chromosomes[$i].gff");
    system("cat ncPOS_$chromosomes[$i].gff >> ncPOS.gff");
    system("$perlsrc/getcommontsc.pl -i nc_UNS_NEG_$chromosomes[$i].gff -r nc_UNS_POS_$chromosomes[$i].gff -o ncUNS_$chromosomes[$i].gff");
    system("cat ncUNS_$chromosomes[$i].gff >> ncUNS.gff");
    system("$perlsrc/getcommontsc.pl -i asUNSNEG_$chromosomes[$i].gff -r nc_UNS_POS_$chromosomes[$i].gff -o asPOS_$chromosomes[$i].gff");
    system("cat asPOS_$chromosomes[$i].gff >> asPOS.gff");
    system("$perlsrc/getcommontsc.pl -i asUNSPOS_$chromosomes[$i].gff -r nc_UNS_NEG_$chromosomes[$i].gff -o asNEG_$chromosomes[$i].gff");
    system("cat asNEG_$chromosomes[$i].gff >> asNEG.gff");
    system("$perlsrc/getcommontsc.pl -i asUNSPOS_$chromosomes[$i].gff -r asUNSNEG_$chromosomes[$i].gff -o undef_$chromosomes[$i].gff");
    system("cat undef_$chromosomes[$i].gff >> undef.gff");
    #>
    
    #< find relative position of ncNEG wrt POS annotated tsc
    system("$perlsrc/posnegcomparechr.pl -i ncNEG_$chromosomes[$i].gff -r POS_tsc_$chromosomes[$i].gff -o ncNEG_$chromosomes[$i]");
    system("cat ncNEG_$chromosomes[$i].included.gff >> ncNEG_included.gff");
    system("cat ncNEG_$chromosomes[$i].intergenic.gff >> ncNEG_intergenic.gff");
    system("cat ncNEG_$chromosomes[$i].overlapped.gff >> ncNEG_overlapped.gff");
    #>
    
    #< find relative position of ncPOS wrt NEG annotated tsc
    system("$perlsrc/posnegcomparechr.pl -i ncPOS_$chromosomes[$i].gff -r NEG_tsc_$chromosomes[$i].gff -o ncPOS_$chromosomes[$i]");
    system("cat ncPOS_$chromosomes[$i].included.gff >> ncPOS_included.gff");
    system("cat ncPOS_$chromosomes[$i].intergenic.gff >> ncPOS_intergenic.gff");
    system("cat ncPOS_$chromosomes[$i].overlapped.gff >> ncPOS_overlapped.gff");
    #>
    
    #< find relative position of ncUNS wrt POS annotated tsc (done wrt NEG while identifying ncUNS)
    system("$perlsrc/posnegcomparechr.pl -i ncUNS_$chromosomes[$i].gff -r POS_tsc_$chromosomes[$i].gff -o ncUNS_$chromosomes[$i]");
    system("cat ncUNS_$chromosomes[$i].included.gff >> ncUNS_included.gff");
    system("cat ncUNS_$chromosomes[$i].intergenic.gff >> ncUNS_intergenic.gff");
    system("cat ncUNS_$chromosomes[$i].overlapped.gff >> ncUNS_overlapped.gff");
    #>

    system("rm \*_$chromosomes[$i]\*gff diff.gff nc.gff");
}

system("cp ncUNS_intergenic.gff UNS_rnaseq.gff");

exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl runtsccompare.pl [--input INPUT] [--genome GENOME] [--src SRCPATH] [--help|?] \n";
    exit 1;
}
