#!/usr/bin/env perl
#!/bin/env perl

## Name: separatestrand.pl
## Version: 1.2.0
## Date: Jan 01, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## separate sam into negative, positive and unstranded reads
# --input, input sam file
# --neg, output negative sam
# --pos, output positive sam
# --uns, output unstranded sam

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $librarytype = "fr-firststrand";
my $seqtype = "single-end";
my $neg = "NEG.sam";
my $pos = "POS.sam";
my $uns = "UNS.sam";

my $line;
my $xs;
my @args;
my %tag;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "library-type=s" => \$librarytype,
			 "seq-type=s" => \$seqtype,
			 "neg=s" => \$neg,
			 "pos=s" => \$pos,
			 "uns=s" => \$uns)
	    or defined $help);

if ($librarytype eq "fr-unstranded") {
    system("cp $input $uns");
    open(NEG, ">$neg");
    open(POS, ">$pos");
    while ($line = <STDIN>) {
        chomp($line);
        @args = split("\t", $line, 2);
        if ($args[0] eq "\@HD" || $args[0] eq "\@SQ" || $args[0] eq "\@PG" || $args[0] eq "\@RG" || $args[0] eq "\@CO") {
            print POS "$line\n";
            print NEG "$line\n";
        }
        else {
            last;
        }
    }
    close(STDIN);
    close(NEG);
    close(POS);
}
else {
    if (defined $input) {
	open(STDIN, $input);
    }
    open(NEG, ">$neg");
    open(POS, ">$pos");
    open(UNS, ">$uns");

    if ($librarytype eq "fr-firststrand") {
	if ($seqtype eq "paired-end") {
	    $tag{99} = "-";	# read paired (2^0) + read mapped in proper pair (2^1) + . + . + . + mate reverse strand (2^5) + first in pair (2^6)
	    $tag{147} = "-";	# read paired (2^0) + read mapped in proper pair (2^1) + . + . + read reverse strand (2^4) + . + . + second in pair (2^7)
	    #$tag{97} = "-";	# 99 - read mapped in proper pair (2^1)
	    #$tag{145} = "-";	# 147 - read mapped in proper pair (2^1)
	    $tag{83} = "+";	# read paired (2^0) + read mapped in proper pair (2^1) + . + . + read reverse strand (2^4) + . + first in pair (2^6)
	    $tag{163} = "+";	# read paired (2^0) + read mapped in proper pair (2^1) + . + . + . + mate reverse strand (2^5) + . + second in pair (2^7)
	    #$tag{81} = "+";	# 83 - read mapped in proper pair (2^1)
	    #$tag{161} = "+";	# 163 - read mapped in proper pair (2^1)
	}
	elsif ($seqtype eq "single-end") {
	    $tag{0} = "-";		
	    $tag{16} = "+";	# read reverse strand (2^4)
	}
    }
    elsif ($librarytype eq "fr-secondstrand") {
	if ($seqtype eq "paired-end") {
	    $tag{99} = "+";
	    $tag{147} = "+";
	    $tag{83} = "-";
	    $tag{163} = "-";
	}
	elsif ($seqtype eq "single-end") {
	    $tag{0} = "+";		
	    $tag{16} = "-";	# read reverse strand (2^4)
	}
    }
    while ($line = <STDIN>) {
	chomp($line);
	@args = split("\t", $line, 2);
	if ($args[0] eq "\@HD" || $args[0] eq "\@SQ" || $args[0] eq "\@PG" || $args[0] eq "\@RG" || $args[0] eq "\@CO") {
	    print POS "$line\n";
	    print NEG "$line\n";
	    print UNS "$line\n";
	}
	else {
	    last;
	}
    }
    if (!$line) {
	exit 0;
    }
    do {	
	chomp($line);
	@args = split("\t", $line, 3);
	
	if ($tag{$args[1]} eq "+") {
	    $xs = "XS:A:-";
   	    if (index($line, $xs) == -1) {	
    		print POS "$line\n";
	    }
	}
	elsif ($tag{$args[1]} eq "-") {
	    $xs = "XS:A:+";
            if (index($line, $xs) == -1) {	    
	        print NEG "$line\n";
	    }	
	}
    } while ($line=<STDIN>);

    close(STDIN);
    close(POS);
    close(NEG);
    close(UNS);
}
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl separatestrand.pl --input GFF [--library-type fr-firststrand|fr-secondstrand|fr-unstranded] [--seq-type single-end|paired-end] [--neg NEG] [--pos POS] [--uns UNS] [--help|?] \n";
    exit 1;
}
